#!/usr/bin/python
# -*- coding: <encoding name> -*-

import logging

kafka_decorator_debug = None

def set_debug_level( val ):
    global kafka_decorator_debug
    kafka_decorator_debug = val

def get_logger( name ):
    global kafka_decorator_debug
    logger = logging.getLogger(name)
    if kafka_decorator_debug is not None:
        logger.setLevel(kafka_decorator_debug)
    return logger 





#!/usr/bin/python
# -*- coding: <encoding name> -*-

from .producer_factory import ProducerFactory
from .logging_helper import get_logger

class Producer:
    def __init__(self, parent, conf):
        self.logger = get_logger(__name__)
        self.logger.info( f"Creating Producer for topic: {conf.topic}" )
        self.__parent__ = parent
        self.__conf__ = conf
        self.__producer__ = None
 
    def produce( self, *func_args, **func_kargs ):
        self.logger.debug(f"Send message: {self.__conf__.topic}" )
        try:
            if self.__producer__ is None:
                self.logger.debug(f"Creating kafka producer: {self.__conf__.topic}" )
                conn =  self.__parent__.getConnection()
                self.__producer__ = ProducerFactory.get_producer( conn, self.__conf__)
            self.__producer__.produce( *func_args, **func_kargs )
            self.logger.debug(f"Mesage sent for topic: {self.__conf__.topic}" )
        except Exception as e:
            self.logger.exception(f"Exception raised: {e}" )
            if self.__producer__ is not None:
                self.__producer__.stop()
                self.__producer__ = None
            raise e
            
    def stop(self):
        self.logger.info( f"Stopping Producer: {self.__conf__.topic}" )
        if self.__producer__ is not None:
            try:
                self.__producer__.stop()
            except Exception as e:
                self.logger.exception( f"Exception on stop listen topic: {self.__conf__.topic} : {type(e)} {e}" )
            self.__producer__ = None
        self.logger.debug( f"producer Stoped: {self.__conf__.topic}" )


#!/usr/bin/python
# -*- coding: <encoding name> -*-

from .kafka import Client
from .kafka.connection_parameter import ConnectionParmeters
from .kafka.consumer_parameter import ConsumerParmeters
from .kafka.producer_parameter import ProducerParmeters
from .kafka.logging_helper import get_logger

class KafkaDecorator:
    def __init__(self):
        self.logger = get_logger(__name__)
        self.logger.info( "Creating decorator " )
        
        self.__topics_receive__ = []
        self.__topics_send__ = []
        self.__connection__ = None
        self.cls = None

    def host(self, *args, **kargs ):
        self.logger.info( f"Adding host" )
        def inner( cls ):
            class NewCls(Client, cls):
                def __init__(obj, *iargs, **ikargs):
                    self.logger.info( f"Creating class: {cls}" )
                    Client.__init__( obj, self.__connection__, self.__topics_receive__, self.__topics_send__ )
                    cls.__init__( obj, *iargs, **ikargs )
                    self.cls = obj
            return NewCls
        self.__connection__ = ConnectionParmeters( args, kargs )
        return inner

    def balanced_consumer(self, topic, *func_args, **func_kargs ):
        self.logger.info( f"Adding balanced consumer, topic: {topic}" )
        def kafka_client_consumer_inner(f):
            c_conf = ConsumerParmeters( ConsumerParmeters.BALANCED, topic, func_args, func_kargs, f )
            self.__topics_receive__.append( c_conf )
            return f
        return kafka_client_consumer_inner
        
    def simple_consumer(self, topic, *func_args, **func_kargs ):
        self.logger.info( f"Adding simple consumer, topic: {topic}" )
        def kafka_client_consumer_inner(f):
            c_conf = ConsumerParmeters( ConsumerParmeters.SIMPLE, topic, func_args, func_kargs, f )
            self.__topics_receive__.append( c_conf )
            return f
        return kafka_client_consumer_inner

    def producer(self, topic, *args, **kargs ):
        self.logger.info( f"Adding producer, topic: {topic}" )
        p_conf = ProducerParmeters(topic, args, kargs)
        self.__topics_send__.append( p_conf )
        def inner_producer( f ): 
            def inner( obj, *func_args, **func_kargs ):
                return self.cls.producer( topic, *func_args, **func_kargs )
            return inner
        return inner_producer


#!/usr/bin/python
# -*- coding: <encoding name> -*-

from .consumer_job import ConsumerJob 
from .producer import Producer
from .logging_helper import get_logger
from time import sleep

from threading import Thread

class Client(Thread):
    def __init__(self, connection_args, list_topics_receive, list_topics_send ):
        self.logger = get_logger(__name__)
        self.logger.info( f"Creating cliet, listen topics: {[t.topic for t in list_topics_receive]} send topics: {[t.topic for t in list_topics_send]}" )
        
        self.__started__  = True
        self.__conumers_failed__ = False
        self.__list_topics_receive__ = list_topics_receive
        self.__connection_args__ = connection_args 
        self.__list_topics_send__ = { p.topic: (p, None) for p in list_topics_send }
        Thread.__init__(self)

    def getConnection(self):
        return self.__connection_args__
        
    def __createConsumers__( self ):
        self.logger.info( "Creating consumers" )
        consumers = []
        
        for conf in self.__list_topics_receive__:
            job = ConsumerJob( self, conf )
            consumers.append( job )
        self.__conumers_failed__ = False
        return consumers
        
    def __startConsumers__( self, consumers ):
        self.logger.info( f"Staring consumers" )
        for c in consumers:
            c.start()
    
    def __stopConsumer__( self, consumers ):
        self.logger.info( f"Stopping consumers" )
        for st in consumers:
            if st is not None:
                st.stop()
            
    def __waitConsumer__( self, consumers ):
        if self.__started__  is False:
            self.__stopConsumer__( consumers )
        
        init = False
        one_failed = False
        for t in consumers:
            if t.is_alive() is True: 
                t.join(0.01)
            if t.is_alive() is True:
                init = True
            else:
                one_failed = True
        if one_failed == True:
            self.__conumers_failed__ = True
            self.__stopConsumer__( consumers )
        return init
                        
    def __waitConsumersFinish__( self, consumers ):
        self.logger.info( f"Waiting consumers finished" )
        init = True
        while init is True:
            init = self.__waitConsumer__( consumers )
        self.logger.info( f"All consumers finished" )
            
    def __waitProducersFinish__( self ):
        self.logger.info( f"Waiting Producers finished" )
        while (not self.__conumers_failed__) and self.__started__ :
            sleep(0.01)
        for p in self.__list_topics_send__.values():
            if p[1] is not None:
                p[1].stop()
        self.logger.info( f"All producers finished" )
                          
    def run(self):
        self.logger.info( f"Start App" )
        self.__started__  = True
        
        while self.__started__  == True:
            sleep(0.1)
            consumers = self.__createConsumers__( )
            self.__startConsumers__( consumers )
            self.__waitConsumersFinish__( consumers )
            self.__waitProducersFinish__( )
        self.logger.info( f"App finished" )

    def producer(self, topic, *func_args, **func_kargs ):
        self.logger.debug( f"Send message to topic{ topic }" )
        pconf, p = self.__list_topics_send__[topic]
        success = True
        try:
            if p is None: 
                p = Producer( self, pconf )
                self.__list_topics_send__[topic] = (pconf, p)
                
            p.produce( *func_args, **func_kargs )
        except (Exception) as e:
            self.logger.exception( f"Cant send topic: {topic} error: {type(e)} {e}" )
            success = False
        return success

    def stop(self):
        self.__started__  = False
        
    def wait(self):
        while self.is_alive( ) is True:
            self.join( 0.01 )
        
    def is_fineshed(self):
        return self.is_alive( ) != True



#!/usr/bin/python
# -*- coding: <encoding name> -*-

from pykafka import KafkaClient

class ConsumerFactory:
    def get_consumer_balanced( conn_args, cons_args  ):
        kafka_client = KafkaClient( *conn_args.args, **conn_args.kargs )
        t = kafka_client.topics[cons_args.topic]
        return t.get_balanced_consumer( *cons_args.args, **cons_args.kargs )
            
    def get_consumer_simple( conn_args, cons_args ):
        kafka_client = KafkaClient( *conn_args.args, **conn_args.kargs )
        t = kafka_client.topics[cons_args.topic]
        return t.get_simple_consumer( *cons_args.args, **cons_args.kargs )
        
	



#!/usr/bin/python
# -*- coding: <encoding name> -*-

class ConsumerParmeters:
    SIMPLE = 1
    BALANCED = 2
    def __init__( self, kind, topic, args, kargs, function ):
        self.args = args
        self.kargs = kargs
        self.function = function
        self.topic = topic
        self.kind = kind


#!/usr/bin/python
# -*- coding: <encoding name> -*-

from pykafka.exceptions import KafkaException
from pykafka.exceptions import ConsumerStoppedException
from threading import Thread
from .consumer_parameter import ConsumerParmeters
from .logging_helper import get_logger
from .consumer_factory import ConsumerFactory 

class ConsumerJob(Thread):
    def __init__(self, parent, conf  ):
        self.logger = get_logger(__name__)
        self.logger.info( f"Creating Consumer for topic: {conf.topic}" )
        
        self.__consumer__ = None
        self.__parent__ = parent
        self.__conf__ = conf
        Thread.__init__(self)

    def __receive__(self, function ):
        self.logger.debug( f"Start receive, topic: {self.__conf__.topic}" )
        for msg in self.__consumer__:
            if msg is not None:
                function( self.__parent__, msg)
                self.logger.debug( f"Message received from topic: {self.__conf__.topic}: offset: {msg.offset}" )
            if self.__started__ == False:
                self.logger.info( f"Asked to stop, topic: {self.__conf__.topic}" )
                break
        self.logger.debug( f"Stop receive, topic: {self.__conf__.topic}" )
    
    def __listen__(self):
        self.logger.info( f"Start listen, topic: {self.__conf__.topic}" )
        try:
            conn  = self.__parent__.getConnection()
            if self.__conf__.kind == ConsumerParmeters.BALANCED:
                self.logger.info( f"Create balanced consumer to topic: {self.__conf__.topic}" )
                self.__consumer__ = ConsumerFactory.get_consumer_balanced( conn,self.__conf__ )
            else:
                self.logger.info( f"Create simple consumer to topic: {self.__conf__.topic}" )
                self.__consumer__ = ConsumerFactory.get_consumer_simple( conn,self.__conf__ )
            f = self.__conf__.function
            while self.__started__ == True:
                self.__receive__( f )
        except ConsumerStoppedException as e:
            self.logger.debug( f"Exception from topic: {self.__conf__.topic} : {type(e)} {e}" )

        self.logger.info( f"Stop listen, topic: {self.__conf__.topic}" )
        
    def run(self):
        self.__started__ = True
        try:
            self.__listen__()
        except KafkaException as e:
            self.logger.exception( f"Exception from topic, when handling another: {self.__conf__.topic} : {type(e)} {e}" )
          
    def stop(self):
        self.logger.info( f"Stopping consumer, topic: {self.__conf__.topic}" )
        if self.__started__ == True:
            self.__started__ = False
            try:
                if self.__consumer__ is not None:
                    self.__consumer__.stop()
                self.logger.debug( f"Stoped consumer, topic: {self.__conf__.topic}" )
            except Exception as e:
                self.logger.exception( f"Exception on stop listen topic: {self.__conf__.topic} : {type(e)} {e}" )
            



import unittest
import mock
import time
from unittest.mock import MagicMock
from pykafka.exceptions import KafkaException
from pykafka.exceptions import ConsumerStoppedException
from kafka_client_decorators.decorators import KafkaDecorator 
from kafka_client_decorators.kafka import ConsumerFactory
from kafka_client_decorators.kafka import ProducerFactory
from kafka_client_decorators.kafka import ProducerParmeters
from kafka_client_decorators.kafka import ConnectionParmeters
from kafka_client_decorators.kafka import set_debug_level
from pykafka import KafkaClient
from threading import Lock 
import logging
 
import logging

class helper_kafka:

    def __init__(self):
        self.stop_execpt = False
        self.read = []
        self.offset = 1
        self.lock = Lock()
    
    def stopException( self ):
        self.stop_execpt = True 
        
    def stop(self):
        if self.stop_execpt is True:
            self.stop_execpt = False
            raise Exception( "Stop error" )
    
    class M:
        def __init__( self, offset, value, partition_key ):
            self.offset = offset
            self.value = value
            self.partition_key = partition_key
        def get(self):
            return self
    class E:
        def __init__( self, excepion ):
            self.excp = excepion

        def get(self):
            raise self.excp
            
    def cleanMessage( self  ):
        with self.lock:
            self.read = []
            self.offset = 1

    def setMessageSemOffset( self, message, key ):
        with self.lock:
            self.read.append( self.M(self.offset, message, key ) )
            self.offset += 1        
       
    def setException( self, excepion  ):
        with self.lock:
            self.read.append( self.E( excepion ) )

    def produce(self, *args, **kargs):
        if args[0] == 'Except'.encode('utf-8'):
            raise Exception("Error in send")
        message =  args[0]
        key = kargs['partition_key'] if 'partition_key' in kargs else None
        self.setMessageSemOffset( message, key )
        
    def __iter__(self):
        return self
    
    def __next__(self):
        time.sleep(0.1)
        if len(self.read) <= 0:
            raise StopIteration
        message = self.read.pop(0)
        return message.get()
        
    def get_producer( self, *args, **kargs ):
        return self
        
    def get_balanced_consumer( self, *args, **kargs ):
        return self
        
    def get_simple_consumer( self, *args, **kargs ):
        return self
          

kc = KafkaDecorator(  )
#@kc.host(zookeeper_hosts='localhost:2181' )
@kc.host(hosts='localhost:9092' )
class A:
    def __init__(self):
        pass

    @kc.producer('test1')
    def sendKey(self, msg, key ):
        pass
        
    @kc.producer('test1')
    def send(self, msg ):
        pass
 
kc2 = KafkaDecorator(  )
@kc2.host(hosts='localhost:9092' )
class B:
    def __init__(self):
        self.read = []
    
    @kc2.balanced_consumer('test1', consumer_group='testgroup3')
    def get(self, msg):
        self.read.append(msg)

kc3 = KafkaDecorator(  )
@kc3.host(hosts='localhost:9092' )
class C:
    def __init__(self):
        self.read = []
    
    @kc3.simple_consumer('test1', consumer_group='testgroup3')
    def get(self, msg):
        self.read.append(msg)

kc4 = KafkaDecorator(  )

@kc4.host(hosts='localhost:9092' )
class D:
    def __init__(self, testA, cls):
        self.a = testA
        self.cls = cls
        pass

    @kc4.balanced_consumer('test1', consumer_group='testgroup3', auto_commit_enable=True, managed=True, consumer_timeout_ms=1000)
    def get(self, msg):
        self.message1 = msg
        self.send( msg.value )

    @kc4.simple_consumer('test2', consumer_group='testgroup4', auto_commit_enable=True, consumer_timeout_ms=1000)
    def get2(self, msg):
        self.message2 = msg
        self.cls.stop(self)

    @kc4.producer('test2')
    def send(self, msg):
        pass

    @kc4.producer('test1')
    def sendKey(self, msg, key ):
        pass

class E:

    def __init__(self):
        pass
        
    def stop(self, conn):
        conn.stop()

class Test1(unittest.TestCase):

    topic1 = helper_kafka()
    topic2 = helper_kafka()
    topic3 = helper_kafka()
    
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_send_key(self):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            set_debug_level(logging.DEBUG)
            self.topic1.cleanMessage()
            
            a = A()
            a.start()

            a.sendKey( 'Hello'.encode('utf-8'), partition_key='world'.encode('utf-8') )
            
            time.sleep(0.5)
            a.stop()
            a.wait()

            assert self.topic1.read[0].offset == 1 
            assert self.topic1.read[0].value == 'Hello'.encode('utf-8') 
            assert self.topic1.read[0].partition_key == 'world'.encode('utf-8')
        
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_start_stop(self):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
        
            a = A()
            a.start()

            a.sendKey( 'Hello'.encode('utf-8'), partition_key='world'.encode('utf-8') )
            assert not a.is_fineshed()
            time.sleep(0.5)
            assert not a.is_fineshed()
            a.sendKey( 'Hello2'.encode('utf-8'), partition_key='world2'.encode('utf-8') )
            assert not a.is_fineshed()
            a.stop()
            a.wait()
            assert a.is_fineshed()
            assert self.topic1.read[0].offset == 1 
            assert self.topic1.read[0].value == 'Hello'.encode('utf-8')
            assert self.topic1.read[0].partition_key == 'world'.encode('utf-8')
            assert self.topic1.read[1].offset == 2 
            assert self.topic1.read[1].value == 'Hello2'.encode('utf-8') 
            assert self.topic1.read[1].partition_key == 'world2'.encode('utf-8') 
    
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_send(self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
            
            a = A()
            a.start()

            a.sendKey( 'Hello'.encode('utf-8') )
            a.stop()
            a.wait()
            
            assert self.topic1.read[0].offset == 1 
            assert self.topic1.read[0].value == 'Hello'.encode('utf-8') 
            assert self.topic1.read[0].partition_key is None 
        
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_exception(self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()

            a = A()
            a.start()

            a.sendKey( 'Except'.encode('utf-8'), partition_key='Except'.encode('utf-8') )

            assert len(self.topic1.read) == 0 
            
            a.sendKey( 'Hello'.encode('utf-8'), partition_key='world'.encode('utf-8') )
            a.stop()
            a.wait()

            assert self.topic1.read[0].offset == 1 
            assert self.topic1.read[0].value == 'Hello'.encode('utf-8') 
            assert self.topic1.read[0].partition_key  == 'world'.encode('utf-8')

        
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_stop_exception(self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
            
            self.topic1.stopException()
        
            a = A()
            a.start()
        
            a.sendKey( 'Erro'.encode('utf-8'), partition_key='Erro'.encode('utf-8') )            
            a.stop()
            a.wait()
            
            assert self.topic1.stop_execpt is False
            
            a = A()
            a.start()
            
            a.sendKey( 'Hello'.encode('utf-8'), partition_key='world'.encode('utf-8') )
            
            a.stop()
            a.wait()
            assert self.topic1.read[0].offset == 1 
            assert self.topic1.read[0].value == 'Erro'.encode('utf-8') 
            assert self.topic1.read[0].partition_key  == 'Erro'.encode('utf-8')
            assert self.topic1.read[1].offset == 2 
            assert self.topic1.read[1].value == 'Hello'.encode('utf-8') 
            assert self.topic1.read[1].partition_key  == 'world'.encode('utf-8')
            
            assert self.topic1.stop_execpt is False
        
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_receive(self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
            
            self.topic1.setMessageSemOffset( 'Hello'.encode('utf-8'), 'world'.encode('utf-8') )
        
            b = B()
            
            b.start()
            b.stop()
            b.wait()

            assert b.read[0].offset == 1
            assert b.read[0].value == 'Hello'.encode('utf-8')
            assert b.read[0].partition_key == 'world'.encode('utf-8')
        
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )
    def test_receive_simple( self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
      
            self.topic1.setMessageSemOffset( 'Hello'.encode('utf-8'), 'world'.encode('utf-8') )
            
            c = C()
            
            c.start()
            c.stop()
            c.wait()
            
            assert c.read[0].offset == 1
            assert c.read[0].value == 'Hello'.encode('utf-8')
            assert c.read[0].partition_key == 'world'.encode('utf-8')
            
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )    
    def test_receive_exception(self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
        
            self.topic1.setMessageSemOffset( 'Hello'.encode('utf-8'), 'world'.encode('utf-8') )
            self.topic1.setMessageSemOffset( 'Hello2'.encode('utf-8'), 'world2'.encode('utf-8') )
            self.topic1.setException( ConsumerStoppedException()  )
            self.topic1.setMessageSemOffset( 'Hello3'.encode('utf-8'), 'world3'.encode('utf-8') )
            
            b = B()
            
            b.start()
            start = time.time()
            while time.time() < (start + 30):
                if len( b.read ) == 3:
                    break
                time.sleep(0.01)
            
            timeout = time.time() > (start + 30)
            
            b.stop()
            b.wait()
            
            assert not timeout
            if timeout :
                return
            
            assert b.read[0].offset == 1
            assert b.read[0].value == 'Hello'.encode('utf-8')
            assert b.read[0].partition_key == 'world'.encode('utf-8')
            
            assert b.read[1].offset == 2
            assert b.read[1].value == 'Hello2'.encode('utf-8')
            assert b.read[1].partition_key == 'world2'.encode('utf-8')
            
            assert b.read[2].offset == 3
            assert b.read[2].value == 'Hello3'.encode('utf-8')
            assert b.read[2].partition_key == 'world3'.encode('utf-8')
        
    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )  
    def test_receive_exception(self ):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
            
            self.topic1.setException( KafkaException())
            self.topic1.setMessageSemOffset( 'Hello'.encode('utf-8'), 'world'.encode('utf-8') )
            self.topic1.setException( KafkaException())
            self.topic1.setMessageSemOffset( 'Hello2'.encode('utf-8'), 'world2'.encode('utf-8') )
            self.topic1.setException( KafkaException())
            self.topic1.setMessageSemOffset( 'Hello3'.encode('utf-8'), 'world3'.encode('utf-8') )
            self.topic1.setException( Exception())
            
            b = B()
            
            b.start()
            start = time.time()
            while time.time() < (start + 30):
                if len( b.read ) == 3:
                    break
                time.sleep(0.01)
            
            timeout = time.time() > (start + 30)
            
            b.stop()
            b.wait()
            
            assert not timeout
            if timeout :
                return
            
            assert b.read[0].offset == 1
            assert b.read[0].value == 'Hello'.encode('utf-8')
            assert b.read[0].partition_key == 'world'.encode('utf-8')
            
            assert b.read[1].offset == 2
            assert b.read[1].value == 'Hello2'.encode('utf-8')
            assert b.read[1].partition_key == 'world2'.encode('utf-8')
            
            assert b.read[2].offset == 3
            assert b.read[2].value == 'Hello3'.encode('utf-8')
            assert b.read[2].partition_key == 'world3'.encode('utf-8')

    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )  
    def test_receive_stop_exception(self):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic1 }
            self.topic1.cleanMessage()
            
            self.topic1.stopException()
            self.topic1.setMessageSemOffset( 'Hello'.encode('utf-8'), 'world'.encode('utf-8') )
            
            b = B()
            
            b.start()
            b.stop()
            b.wait()
            
            assert b.read[0].offset == 1
            assert b.read[0].value == 'Hello'.encode('utf-8')
            assert b.read[0].partition_key == 'world'.encode('utf-8')
            
            self.topic1.setMessageSemOffset( 'Hello2'.encode('utf-8'), 'world2'.encode('utf-8') )
            
            b = B()
            
            b.start()
            b.stop()
            b.wait()
            
            assert b.read[0].offset == 2
            assert b.read[0].value == 'Hello2'.encode('utf-8')
            assert b.read[0].partition_key == 'world2'.encode('utf-8')

    @mock.patch.object( KafkaClient, '__init__', lambda self, *args, **kargs: None )  
    def test_send_receive_many(self):
        with mock.patch('pykafka.KafkaClient.topics', new_callable=mock.PropertyMock, create=True) as mock_foo:
            mock_foo.return_value = {'test1': self.topic2,  'test2': self.topic3}
            self.topic1.cleanMessage()
            self.topic2.cleanMessage()

            a = D('Example', E())
            a.start()
            a.sendKey( 'Hello'.encode('utf-8'), partition_key='world'.encode('utf-8') )
            a.wait() 
            
            assert a.message1.offset == 1
            assert a.message1.value == 'Hello'.encode('utf-8')
            assert a.message1.partition_key == 'world'.encode('utf-8')
            
            assert a.message1.offset == 1
            assert a.message1.value == 'Hello'.encode('utf-8')
            assert a.message1.partition_key == 'world'.encode('utf-8')
            
if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: <encoding name> -*-

from .client import Client
from .producer_factory import ProducerFactory
from .consumer_factory import ConsumerFactory
from .consumer_parameter import ConsumerParmeters
from .producer_parameter import ProducerParmeters
from .connection_parameter import ConnectionParmeters
from .logging_helper import  set_debug_level





#!/usr/bin/python
# -*- coding: <encoding name> -*-

class ProducerParmeters:
    def __init__( self, topic, args, kargs ):
        self.args = args
        self.kargs = kargs
        self.topic = topic
        


#!/usr/bin/python
# -*- coding: <encoding name> -*-

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    required = [x.strip() for x in f.read().splitlines()]

with open('requirements_test.txt') as f:
    required_test = [x.strip() for x in f.read().splitlines()]

setuptools.setup(
    name="kafka_client_decorators",
    version="0.9.5",
    author="Edson Cardoso",
    author_email="edsonsn2@hotmail.com",
    description="Decorator interface to pykafka",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cdsedson/kafka-decorator.git",
    packages=setuptools.find_packages( exclude=['tests','tests.*'] ),
    extras_require = {
        'test': required_test
        },
    test_suite = 'nose.collector',
    install_requires=required,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



#!/usr/bin/python
# -*- coding: <encoding name> -*-

from pykafka import KafkaClient

class ProducerFactory:
    def get_producer( conn_args, prod_args  ):
         kafka_client = KafkaClient( *conn_args.args, **conn_args.kargs)
         t = kafka_client.topics[prod_args.topic] 
         return t.get_producer( *prod_args.args, **prod_args.kargs )
        
	